FRISK_API_KEY = "FRISK_API_KEY"

FRISK_BASE_URL = "FRISK_BASE_URL"
FRISK_BASE_URL_DEFAULT = "https://api.friskai.com"

FRISK_TELEMETRY_ENDPOINT = "FRISK_TELEMETRY_ENDPOINT"
FRISK_TELEMETRY_ENDPOINT_DEFAULT = "https://ingest.friskai.com"

# If set to TRUE, sets the log level to DEBUG inside Frisk.
FRISK_DEBUG = "FRISK_DEBUG"
FRISK_DEBUG_DEFAULT = "false"

FRISK_SHUTDOWN_TIMEOUT_S = "FRISK_SHUTDOWN_TIMEOUT_S"
FRISK_SHUTDOWN_TIMEOUT_S_DEFAULT = 5
